import _extends from "@babel/runtime/helpers/esm/extends";
import { useReducer, useRef } from 'react';
import Domain from '../domain/index.js';
import useDomainEventSubscriptions from './useDomainEventSubscriptions.js';
var domain = new Domain();
var ACTIONS = {
  ENABLE_DEV_MODE: 'ENABLE_DEV_MODE',
  SET_STATE: 'SET_STATE',
  TOOGLE_TAB: 'toggleTab'
};
var initialState = {
  tasks: [],
  toggleTab: false,
  isDevModeEnabled: false
};
var reducer = function reducer(state, action) {
  switch (action.type) {
    case ACTIONS.ENABLE_DEV_MODE:
      return _extends({}, state, {
        isDevModeEnabled: true
      });
    case ACTIONS.SET_STATE:
      return _extends({}, state, action.payload);
    case ACTIONS.TOOGLE_TAB:
      return _extends({}, state, {
        toggleTab: !state.toggleTab
      });
    default:
      return state;
  }
};

// Create a custom hook that uses the reducer and returns the state and dispatch function
var useState = function useState(_ref) {
  var onCompleteAllTasks = _ref.onCompleteAllTasks;
  var _useReducer = useReducer(reducer, _extends({}, initialState)),
    state = _useReducer[0],
    dispatch = _useReducer[1];
  var stateRef = useRef(state);
  stateRef.current = state;
  var setState = function setState(state) {
    return dispatch({
      type: ACTIONS.SET_STATE,
      payload: state
    });
  };
  var toggleTab = function toggleTab() {
    return dispatch({
      type: ACTIONS.TOOGLE_TAB
    });
  };
  var executeUseCase = function executeUseCase(useCaseName, params, next) {
    if (next === void 0) {
      next = function next() {
        return null;
      };
    }
    return domain.get(useCaseName).execute(params).then(function (result) {
      setState(result);
      next(result);
    });
  };
  var checkIfAllTasksHaveBeenFinished = function checkIfAllTasksHaveBeenFinished(state) {
    var completedTasks = countCompletedTasks(state);
    var allTasks = countTasks(state);
    if (completedTasks === allTasks && allTasks > 0) onCompleteAllTasks(state);
  };
  useDomainEventSubscriptions(domain, executeUseCase, onCompleteAllTasks);
  var runSimpleTask = function runSimpleTask(task) {
    return executeUseCase('run_simple_task_use_case', _extends({
      localState: stateRef.current
    }, task));
  };
  var runTask = function runTask(task) {
    return executeUseCase('run_task_use_case', _extends({
      localState: stateRef.current
    }, task));
  };
  var setPercentage = function setPercentage(taskId, workId, percentage) {
    return executeUseCase('set_work_percentage_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId,
      percentage: percentage
    });
  };
  var finishWork = function finishWork(taskId, workId, result) {
    if (result === void 0) {
      result = null;
    }
    return executeUseCase('finish_work_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId,
      result: result
    }, checkIfAllTasksHaveBeenFinished);
  };
  var cancelWork = function cancelWork(taskId, workId) {
    return executeUseCase('cancel_work_task_use_case', {
      localState: stateRef.current,
      taskId: taskId,
      workId: workId
    });
  };
  var errorWork = function errorWork(taskId, workId, log) {
    return executeUseCase('error_work_task_use_case', {
      localState: stateRef.current,
      log: log,
      taskId: taskId,
      workId: workId
    });
  };
  var enableDevMode = function enableDevMode() {
    return dispatch({
      type: ACTIONS.ENABLE_DEV_MODE
    });
  };

  // Getters
  // TODO: Most of this functions will be migrated to domain soon
  var getState = function getState() {
    return stateRef.current;
  };
  var getTask = function getTask(taskId, state) {
    if (state === void 0) {
      state = getState();
    }
    return state.tasks.find(function (task) {
      return task.id === taskId;
    });
  };
  // Work status check
  var _isVisibleWork = function _isVisibleWork(work, state) {
    if (state === void 0) {
      state = getState();
    }
    return work.isVisible || state.isDevModeEnabled;
  };
  var _isFinishedWork = function _isFinishedWork(work) {
    return work.status === domain.get('config').get('AVAILABLE_STATUS').COMPLETED;
  };
  var _isErroredWork = function _isErroredWork(work) {
    return work.status === domain.get('config').get('AVAILABLE_STATUS').ERROR;
  };
  // Counts the number of work that meets a specific criteria
  var _countWorkFromTask = function _countWorkFromTask(taskId, countCriteria, state) {
    if (state === void 0) {
      state = getState();
    }
    var task = getTask(taskId, state);
    if (task === undefined) return 0;
    return task.work.filter(countCriteria).length;
  };

  // Count visible work from a specific task
  var countWork = function countWork(taskId, state) {
    if (state === void 0) {
      state = getState();
    }
    return _countWorkFromTask(taskId, function (work) {
      return _isVisibleWork(work, state);
    }, state);
  };
  // Count visible and finished work from a specific task
  var countFinishedWork = function countFinishedWork(taskId, state) {
    if (state === void 0) {
      state = getState();
    }
    return _countWorkFromTask(taskId, function (work) {
      return _isVisibleWork(work, state) && _isFinishedWork(work);
    });
  };
  var countErroredWork = function countErroredWork(taskId) {
    return _countWorkFromTask(taskId, function (work) {
      return _isErroredWork(work);
    });
  };

  // Checks if a task is visible
  var _isVisibleTask = function _isVisibleTask(taskId, state) {
    if (state === void 0) {
      state = getState();
    }
    return countWork(taskId, state) > 0;
  };
  // Checks if a task has errored work
  var _isErroredTask = function _isErroredTask(taskId) {
    return countErroredWork(taskId) > 0;
  };
  // Counts the number of tasks that are being processed
  var countInProgressTasks = function countInProgressTasks() {
    return getState().tasks.filter(function (task) {
      return _isVisibleTask(task.id) && countWork(task.id) > countFinishedWork(task.id) && _isErroredTask(task.id) === false;
    }).length;
  };
  // Counts the number of tasks that finished
  var countCompletedTasks = function countCompletedTasks(state) {
    if (state === void 0) {
      state = getState();
    }
    return state.tasks.filter(function (task) {
      return _isVisibleTask(task.id, state) && countWork(task.id, state) === countFinishedWork(task.id, state);
    }).length;
  };

  // Counts the number of tasks that are visible, regardless of their status
  var countTasks = function countTasks(state) {
    if (state === void 0) {
      state = getState();
    }
    return state.tasks.filter(function (task) {
      return _isVisibleTask(task.id, state);
    }).length;
  };
  var getInProgressTaskPercentage = function getInProgressTaskPercentage() {
    var inProgressTask = getState().tasks.find(function (task) {
      return task.status === domain.get('config').get('AVAILABLE_STATUS').IN_PROGRESS;
    });
    if (inProgressTask === undefined) return 0;
    var work = countWork(inProgressTask.id);
    var finishedWork = countFinishedWork(inProgressTask.id);
    var inProgressWork = inProgressTask.work.find(function (work) {
      return work.status === domain.get('config').get('AVAILABLE_STATUS').IN_PROGRESS;
    });
    var inProgressWorkPercentage = 0;
    if (inProgressWork !== undefined) {
      inProgressWorkPercentage = inProgressWork.percentage;
    }
    var completionPercentage = Math.round((finishedWork + inProgressWorkPercentage / 100) * 100 / work);
    return completionPercentage;
  };
  return {
    cancelWork: cancelWork,
    domain: domain,
    enableDevMode: enableDevMode,
    errorWork: errorWork,
    finishWork: finishWork,
    getState: getState,
    getTask: getTask,
    runSimpleTask: runSimpleTask,
    runTask: runTask,
    setPercentage: setPercentage,
    toggleTab: toggleTab,
    /* getters */
    countFinishedWork: countFinishedWork,
    countInProgressTasks: countInProgressTasks,
    countTasks: countTasks,
    countWork: countWork,
    getInProgressTaskPercentage: getInProgressTaskPercentage
  };
};
export default useState;